package java.util;

import java.lang.invoke.SerializedLambda;
import java.io.Serializable;
import java.util.function.Function;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;

public interface Map<K, V>
{
    public abstract int size();
    
    public abstract boolean isEmpty();
    
    public abstract boolean containsKey(Object p0);
    
    public abstract boolean containsValue(Object p0);
    
    public abstract V get(Object p0);
    
    public abstract V put(K p0, V p1);
    
    public abstract V remove(Object p0);
    
    public abstract void putAll(Map<? extends K, ? extends V> p0);
    
    public abstract void clear();
    
    public abstract Set<K> keySet();
    
    public abstract Collection<V> values();
    
    public abstract Set<Map.Entry<K, V>> entrySet();
    
    public abstract boolean equals(Object p0);
    
    public abstract int hashCode();
    
    default public V getOrDefault(Object o, V v) {
        Object value;
        return ((value = this.get(o)) != null || this.containsKey(o)) ? value : v;
    }
    
    default public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(biConsumer);
        for (Map.Entry entry : this.entrySet()) {
            Object key;
            Object value;
            try {
                key = entry.getKey();
                value = entry.getValue();
            }
            catch (IllegalStateException ex) {
                throw new ConcurrentModificationException(ex);
            }
            biConsumer.accept(key, value);
        }
    }
    
    default public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        for (Map.Entry entry : this.entrySet()) {
            Object key;
            Object value;
            try {
                key = entry.getKey();
                value = entry.getValue();
            }
            catch (IllegalStateException ex) {
                throw new ConcurrentModificationException(ex);
            }
            Object apply = biFunction.apply(key, value);
            try {
                entry.setValue(apply);
            }
            catch (IllegalStateException ex2) {
                throw new ConcurrentModificationException(ex2);
            }
        }
    }
    
    default public V putIfAbsent(K k, V v) {
        Object o = this.get(k);
        if (o == null) {
            o = this.put(k, v);
            return o;
        }
        return o;
    }
    
    default public boolean remove(Object o, Object o2) {
        Object value = this.get(o);
        if (!Objects.equals(value, o2) || (value == null && !this.containsKey(o))) {
            return false;
        }
        this.remove(o);
        return true;
    }
    
    default public boolean replace(K k, V v, V v2) {
        Object value = this.get(k);
        if (!Objects.equals(value, v) || (value == null && !this.containsKey(k))) {
            return false;
        }
        this.put(k, v2);
        return true;
    }
    
    default public V replace(K k, V v) {
        Object o;
        if ((o = this.get(k)) != null || this.containsKey(k)) {
            o = this.put(k, v);
            return o;
        }
        return o;
    }
    
    default public V computeIfAbsent(K k, Function<? super K, ? extends V> function) {
        Objects.requireNonNull(function);
        Object value;
        Object apply;
        if ((value = this.get(k)) == null && (apply = function.apply(k)) != null) {
            this.put(k, apply);
            return apply;
        }
        return value;
    }
    
    default public V computeIfPresent(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Object value;
        if ((value = this.get(k)) != null) {
            Object apply = biFunction.apply(k, value);
            if (apply != null) {
                this.put(k, apply);
                return apply;
            }
            this.remove(k);
            return null;
        }
        else {
            return null;
        }
    }
    
    default public V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Object value = this.get(k);
        Object apply = biFunction.apply(k, value);
        if (apply == null) {
            if (value != null || this.containsKey(k)) {
                this.remove(k);
                return null;
            }
            return null;
        }
        else {
            this.put(k, apply);
            return apply;
        }
    }
    
    default public V merge(K k, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Objects.requireNonNull(v);
        Object value = this.get(k);
        Object o = (value == null) ? v : biFunction.apply(value, v);
        if (o == null) {
            this.remove(k);
            return o;
        }
        this.put(k, o);
        return o;
    }
    
    public interface Entry<K, V>
    {
        public abstract K getKey();
        
        public abstract V getValue();
        
        public abstract V setValue(V p0);
        
        public abstract boolean equals(Object p0);
        
        public abstract int hashCode();
        
        public static <K extends Comparable<? super K>, V> Comparator<Map.Entry<K, V>> comparingByKey() {
            return (Comparator)(Serializable)((entry, entry2)/* java.util.Map.Entry,java.util.Map.Entry, */ -> ((Comparable)entry.getKey()).compareTo(entry2.getKey()));
        }
        
        public static <K, V extends Comparable<? super V>> Comparator<Map.Entry<K, V>> comparingByValue() {
            return (Comparator)(Serializable)((entry, entry2)/* java.util.Map.Entry,java.util.Map.Entry, */ -> ((Comparable)entry.getValue()).compareTo(entry2.getValue()));
        }
        
        public static <K, V> Comparator<Map.Entry<K, V>> comparingByKey(Comparator<? super K> comparator) {
            Objects.requireNonNull(comparator);
            return (Comparator)(Serializable)((entry, entry2)/* java.util.Map.Entry,java.util.Map.Entry, */ -> p0.compare(entry.getKey(), entry2.getKey()));
        }
        
        public static <K, V> Comparator<Map.Entry<K, V>> comparingByValue(Comparator<? super V> comparator) {
            Objects.requireNonNull(comparator);
            return (Comparator)(Serializable)((entry, entry2)/* java.util.Map.Entry,java.util.Map.Entry, */ -> p0.compare(entry.getValue(), entry2.getValue()));
        }
    }
}
